/* Figure 1 - all panels */

// Select panel of Fig 1 to be generated (1=a,...a,5=e)
#define Panel 1

// Some constants used in all models
#define AGINGRATE 1 // aging rate: a constant except for Model e
#define dt 0.25 	// time increment per simulatin step
#define STEPS 1202	// number of simulation steps

#define tau 0.5	  // controls symmetry of cell divisions, modified in model e.
#define amax 80.5 // max cell age when division controlled by age (Panels a,b,c)
#define xmax 1	  // max cell size when division size (Panels d,e).

Lsystem: 1
derivation length: STEPS-1
Start: {
	// Constants used in specific models, chosen to produce aesthetic figures. 
	if(Panel != 5) {mtau=0;} else {mtau = 0.075; srand(19);}
	if(Panel==1 || Panel==2 || Panel==5) {RERG=0.0086;} else {RERG=0.006;} 
	if(Panel!=4) {CELL_AGE_TO_COLOR=1.3;} else {CELL_AGE_TO_COLOR=1.3*0.697;}
	yinit = 7; dy = yinit/STEPS; 
	n=0;
}

StartEach: {n=n+1;}
Axiom: f(yinit) V(0) -(90) [W(0) C(tau,0,AGINGRATE) W(0)]

// Cells get older
C(x,a,ar) : n<STEPS-1 --> C(x*(1+RERG*dt),a+ar*dt,ar)
// Walls get older
W(age) : n<STEPS-1 --> W(age+AGINGRATE*dt)

// Draw consecutive stages one below another
V(y) : n<STEPS-1 --> V(y - dy)
// Draw final stage significantly further down
V(h) --> V(h-0.4)

decomposition
// Cell division

// Panels a, b (time version), c: cell division controlled by cell age
C(x,a,ar) : a > 80.5 && (Panel==1 || Panel==2 || Panel==3) --> C(x*tau,0,ar) W(0) C(x*(1-tau),0,ar)

// Panels d: division controlled by cell size (genereal case of e, kept because of simplicity )
C(x,a,ar) : x > xmax && (Panel==4)--> C(x*tau,0,ar) W(0) C(x*(1-tau),0,ar) // Panel b1-space,d

// Panel e: division controlled by cell size with a rendom factor
C(x,a,ar) : x > xmax && (Panel==5) {rtau = tau*(1-mtau + ran(2*mtau)); } --> C(x*rtau,0,AGINGRATE*log(0.5)/log(rtau)) W(0) // Panel e
			C(x*(1-rtau),0,AGINGRATE*log(0.5)/log(1-rtau)) // Panel e

homomorphism
maximum depth: 6

V(y) --> f(y)

// All panels except a : use continuous representation of cell states. 
C(x,a,ar) : Panel != 1 && n != STEPS-1 --> ;(16+a*CELL_AGE_TO_COLOR) F(x) 

// Panel a or final state : show individual cells 
W(agel) < C(s,a,ar) > W(ager) : n%115 == 2 || n==STEPS-1 
	{l=s; w=0.4; bl=0.5*(0.05 + 0.0004*agel); br=0.5*(0.05 + 0.0004*ager); bw=0.04; r=0.05; c=16+a*CELL_AGE_TO_COLOR;} -->
	;(255) R(l,w,r) [f(-l) ^ f(0.1) & ;(c) f(bl) R(l-(bl+br), w-2*bw, r-bw/2)] f(0.000)

// Cell boundary
R(l,w,r) : 1 {h = 2*r*1.41421;} --> [ !(1) - f((w-h)/2) |
	   { F(w-h) -(45) F(r) [- f(r) @c(h)] F(r) -(45)
	     F(l-h) -(45) F(r) [- f(r) @c(h)] F(r) -(45)
	     F(w-h) -(45) F(r) [- f(r) @c(h)] F(r) -(45)
	     F(l-h) -(45) F(r) [- f(r) @c(h)] F(r) -(45) }  ] f(l)

// All panels except a : use continuous representation of cell walls.
W(age) : Panel==1 --> *
C(xl,al,arl) < W(age) > C(xr,ar,arr) --> [;(255) ^(90)f(1)&(90) @c(0.05 + 0.0004*age)]
W(age) > C(xr,ar,arr) --> [;(255) ^(90)f(1) &(90) f( 0.20*(0.05 + 0.0004*age)) @c(0.5*(0.05 + 0.0004*age))]
C(xl,al,arl) < W(age) --> [;(255) ^(90)f(1) &(90) f(-0.20*(0.05 + 0.0004*age)) @c(0.5*(0.05 + 0.0004*age))]

endlsystem
